"""
Static and important data which is needed by the program and do not need any third party libraries. (this is important
because it is used inside the setup.py **| do not edit**.
"""

#: name of this program
NAME = "unidown"
#: long name of this program
LONG_NAME = "Universal Downloader"
#: version in PEP440 format
VERSION = '2.0.0.dev7'
#: url to the unidown pypi json
PYPI_JSON_URL = 'https://pypi.python.org/pypi/unidown/json'
#: author
AUTHOR = 'Iceflower S'
#: author github link
AUTHOR_GITHUB = 'https://github.com/IceflowRE'
#: author email
AUTHOR_EMAIL = 'iceflower@gmx.de'
#: short description
DESCRIPTION = 'Universal downloader, a modular extensible downloader who manage progress and updates.'
#: project url
PROJECT_URL = 'https://github.com/IceflowRE/unidown'
